
precision highp float;

uniform sampler2D texture; /* ベーステクスチャー */
uniform sampler2D CC_Texture0;
uniform vec4 tintColor;    /* 文字色 */
uniform vec4 texGeom;      /* vec4(texture.width, texture.height, texture.x, texture.y) */

uniform vec4 outline_color;
uniform vec4 shadow_color;
uniform vec2 outline_shape;
uniform vec2 shadow_shape;
uniform float outline_band;
uniform float shadow_band;
uniform float shadow_edge;

/* ベーステクスチャーのアルファを求める。アンチエイリアスのかかっている場所では 0 と 1 の間を取る */
float baseAlpha(vec2 coord)
{
  return texture2D(CC_Texture0, vec2(coord.x / texGeom.x, coord.y / texGeom.y)).a;
}
/* 下の色と上の色をアルファつきで混ぜる */
vec4 mergeOpaqueColors(vec4 under, vec4 over)
{
  vec4 ret=under;
  if (over.a != 0.0){
	  float under_a = under.a * (1.0 - over.a);
	  float a = over.a + under_a;
	  ret = vec4((over.rgb * over.w + under.rgb * under_a) / a, a);
  }
  return ret;
}

float checkX(vec2 center,float x,float y,float min){
	float ret = min;
	vec2 xy = vec2(x, y);
	float a = baseAlpha(center+xy);
	if (a != 0.0){
		float r = length(xy)+ 1.0 - a;
		if (r<min){
			ret = r;
		}
	}
	return ret;
}

vec4 mergeShadow(vec4 lastColor, vec4 sc, vec2 d, float w ,float e)
{
  vec2 center = gl_FragCoord.xy - texGeom.zw + d;

  float r1=e;
  float r0=w;
  float rMin=r0;
  float maxY= floor(r0-1.0);
  float minY= -maxY;

  for (float y=-2.0;y<=2.0;y+=1.0){
	if ((y<maxY)&&(y>minY)){

	    float r_1 = rMin - 1.0;

//		rMin = checkX(center,-5.0,y,rMin);
//		rMin = checkX(center,-4.0,y,rMin);
//		rMin = checkX(center,-3.0,y,rMin);
		rMin = checkX(center,-2.0,y,rMin);
		rMin = checkX(center,-1.0,y,rMin);
		rMin = checkX(center,-0.0,y,rMin);
		rMin = checkX(center, 1.0,y,rMin);
		rMin = checkX(center, 2.0,y,rMin);
//		rMin = checkX(center, 3.0,y,rMin);
//		rMin = checkX(center, 4.0,y,rMin);

	}
  }

  if (r0 <= rMin) return lastColor; // 影の外側 
  vec4 cc=sc;
  cc.a = cc.a * (1.0 - smoothstep(r1, r0, rMin));
  return mergeOpaqueColors(lastColor, cc);
}



void main(void)
{
  vec4 color  = vec4(0.0, 0.0, 0.0, 0.0);

//  outline_color = vec4(0.0, 0.0, 1.0, 1.0);
//  shadow_color= vec4(1.0, 0.0, 0.0, 1.0);
//  outline_shape =vec2(0.0,0.0);
//  shadow_shape=vec2(3.0,-3.0);
//  outline_band=4.0;
//  shadow_band=6.0;

  color = mergeShadow(color,shadow_color,shadow_shape,shadow_band,shadow_edge); // 影
  color = mergeShadow(color,outline_color,outline_shape,outline_band,outline_band); // アウトライン
  float face_a = baseAlpha(gl_FragCoord.xy - texGeom.zw);
  if (0.0 != face_a) {
    vec4 innerColor = tintColor;
    innerColor.a *= face_a;
    color = mergeOpaqueColors(color, innerColor); // 元フォント
  }
  /* 最終出力 */
  color.a=sqrt(color.a);
  gl_FragColor = color;

}
